use super::source::Source;

#[derive(Debug)]
pub enum Object<'s> {
    Bold(Bold<'s>),
    Italic(Italic<'s>),
    Underline(Underline<'s>),
    StrikeThrough(StrikeThrough<'s>),
    Code(Code<'s>),
    Verbatim(Verbatim<'s>),

    PlainText(PlainText<'s>),

    #[allow(dead_code)]
    RegularLink(RegularLink<'s>),
}

#[derive(Debug)]
pub struct Bold<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug)]
pub struct Italic<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug)]
pub struct Underline<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug)]
pub struct StrikeThrough<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug)]
pub struct Code<'s> {
    pub source: &'s str,
    pub contents: &'s str,
}

#[derive(Debug)]
pub struct Verbatim<'s> {
    pub source: &'s str,
    pub contents: &'s str,
}

#[derive(Debug)]
pub struct PlainText<'s> {
    pub source: &'s str,
}

#[derive(Debug)]
pub struct RegularLink<'s> {
    pub source: &'s str,
}

impl<'s> Source<'s> for Object<'s> {
    fn get_source(&'s self) -> &'s str {
        match self {
            Object::Bold(obj) => obj.source,
            Object::Italic(obj) => obj.source,
            Object::Underline(obj) => obj.source,
            Object::StrikeThrough(obj) => obj.source,
            Object::Code(obj) => obj.source,
            Object::Verbatim(obj) => obj.source,
            Object::PlainText(obj) => obj.source,
            Object::RegularLink(obj) => obj.source,
        }
    }
}

impl<'s> Source<'s> for Bold<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Italic<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Underline<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for StrikeThrough<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Code<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Verbatim<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
